package com.pigbite.cdh.generation;

/**
 * This generator extends the base DirectoryGenerator to cover the
 * specifics of ZIP files.
 *
 * @author <a href="mailto:jez@pigbite.com">Jeremy Aston</a>
 * @version 1.00
 */

import java.util.Date;
import java.util.Enumeration;
import java.util.Vector;
import java.util.zip.*;
import org.apache.cocoon.generation.*;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;
import java.io.File;
import java.io.IOException;

/**
 * Generates a directory listing that displays extended information
 * for ZIP files.
 */
public class ZipDirectoryGenerator extends DirectoryGenerator
{

	// Properties
		
	/** Number of entries in the ZIP file **/
	private int numEntries = 0;
	
	/** Entries **/
	private Vector entries = null;

	// Nodes
    /** ZipEntry */
    protected static String ZIP_FILE_ZIPENTRY_NODE_NAME = "zipEntry";
		
    // Attributes
    /** Number of entries in the zip file attribute name */
    protected static String ZIP_FILE_NUM_ENTRIES_ATTR_NAME = "numEntries";
    /** Entry file name attribute name */
    protected static String ZIP_ENTRY_NAME_ATTR_NAME = "name";
    /** Compressed size attribute name */
    protected static String ZIP_ENTRY_COMP_SIZE_ATTR_NAME = "compressedSize";
    /** Compression method attribute name */
    protected static String ZIP_ENTRY_COMP_METHOD_ATTR_NAME = "method";
    /** Stored (uncompressed) Compression method text value */
    protected static String ZIP_ENTRY_STORED_TEXT = "stored";
    /** Deflated (compressed) Compression method text value */
    protected static String ZIP_ENTRY_DEFLATED_TEXT = "deflated";
    /** Unknown Compression method text value */
    protected static String ZIP_ENTRY_UNKNOWN_TEXT = "unknown";
    /** Uncompressed size attribute name */
    protected static String ZIP_ENTRY_UNCOMP_SIZE_ATTR_NAME = "uncompressedSize";
    /** CRC32 Checksum attribute name */
    protected static String ZIP_ENTRY_CRC_ATTR_NAME = "crc32Checksum";
    /** Comment attribute name */
    protected static String ZIP_ENTRY_COMMENT_ATTR_NAME = "comment";
    /** Extra Data attribute name */
    protected static String ZIP_ENTRY_EXTRA_DATA_ATTR_NAME = "extraData";
    /** Modification time attribute name */
    protected static String ZIP_ENTRY_MODIFIED_ATTR_NAME = "lastModified";
    /** Modification time attribute name */
    protected static String ZIP_ENTRY_DATE_ATTR_NAME = "date";
    /** Directory indicator attribute name */
    protected static String ZIP_ENTRY_DIR_ATTR_NAME = "isDirectory";


    /**
     * Extends the <code>setNodeAttributes</code> method from the
     * <code>DirectoryGenerator</code> by adding ZIP file information
     * assuming the ZIP file is valid,
     * 
     * @param   path
     *      the path being looked at
     */
    protected void setNodeAttributes(File path) throws SAXException
    {

		// Call the DirectoryGenerator method to get all the basic
		// node attributes
        super.setNodeAttributes( path );

		// If we are looking at a directory then move on cos
		// there is nothing to see here,,,
        if (path.isDirectory()) {
            return;
        }


		ZipFile zip = null;
		
		try
		{

			getLogger().debug( "Attempting to open ZIP file " + path );
			
			// Open the zip file
			zip = new ZipFile( path );
			
			// Add the number of entries as an attribute
			this.numEntries = zip.size();
			attributes.addAttribute( "", ZIP_FILE_NUM_ENTRIES_ATTR_NAME, ZIP_FILE_NUM_ENTRIES_ATTR_NAME, "CDATA", "" + this.numEntries );
			
			// Save the entries.  This is because otherwise the entries appear
			// before the parent node.
			this.entries = new Vector();
			
			for ( Enumeration e = zip.entries(); e.hasMoreElements() ; )
			{
				this.entries.add( (ZipEntry)e.nextElement() );
			}
			
		}
		catch ( ZipException ze )
		{
			getLogger().debug( "ZIPException opening ZIP file " + path, ze );
		}
		catch ( IOException ioe )
		{
			getLogger().debug( "IOException opening ZIP file " + path, ioe );
		}
		finally
		{
		
			// Close the ZIP file if it was sucessfully opened
				
			if ( zip != null )
			{
				try
				{
					zip.close();
				}
				catch ( IOException ioe )
				{
					getLogger().debug( "IOException closing ZIP file " + path, ioe );
				}
			}
			
		}

    }
    
    /**
     * Ends the named node. Overrides the DirectoryGenerator endNode.  This
     * is so that the zipEntry nodes can be placed in the correct place
     *
     * @param   nodeName
     *      the name of the new node
     * @param   path
     *      the file/directory to use when setting attributes
     *
     * @throws  SAXException
     *      if an error occurs while closing the node
     */
    protected void endNode(String nodeName)
    throws SAXException {

		// List any zip file entries and then clear them to
		// prevent them displaying on any irrelevant nodes.
		listEntries();    	
		this.entries.clear();
		
    	super.endNode( nodeName );

    }


	/**
	 * Method listEntries.
	 * Iterates through an enumeration of ZipEntries and adds nodes describing
	 * each one.
	 * 
	 * @param entries
	 * @throws SAXException
	 */
    private void listEntries( )
    throws SAXException
    {
    	
		Enumeration e = null;
		
		if ( this.entries == null )
		{
			return;
		}

		e = this.entries.elements();
		
		getLogger().debug( "Doing entries" );
		
		
		while ( e.hasMoreElements() )
		{
			ZipEntry zipEntry = (ZipEntry)e.nextElement();
			getLogger().debug( "Doing entry: " + zipEntry.getName() );
			
			AttributesImpl attributes = new AttributesImpl();
						
			// Entry Name
			attributes.addAttribute( URI, ZIP_ENTRY_NAME_ATTR_NAME, 
						ZIP_ENTRY_NAME_ATTR_NAME, "CDATA", 
						zipEntry.getName() );
			// Compressed Size
			attributes.addAttribute( URI, ZIP_ENTRY_COMP_SIZE_ATTR_NAME, 
						ZIP_ENTRY_COMP_SIZE_ATTR_NAME, "CDATA", 
						"" + zipEntry.getCompressedSize() );

			// Compression method
			int method = zipEntry.getMethod();
			
			switch ( zipEntry.getMethod() )
			{
				case ZipEntry.DEFLATED :
					attributes.addAttribute( URI, ZIP_ENTRY_COMP_METHOD_ATTR_NAME, 
								ZIP_ENTRY_COMP_METHOD_ATTR_NAME, "CDATA", 
								ZIP_ENTRY_DEFLATED_TEXT );
					break;
				case ZipEntry.STORED :
					attributes.addAttribute( URI, ZIP_ENTRY_COMP_METHOD_ATTR_NAME, 
								ZIP_ENTRY_COMP_METHOD_ATTR_NAME, "CDATA", 
								ZIP_ENTRY_STORED_TEXT );
					break;
				default :
					attributes.addAttribute( URI, ZIP_ENTRY_COMP_METHOD_ATTR_NAME, 
								ZIP_ENTRY_COMP_METHOD_ATTR_NAME, "CDATA", 
								ZIP_ENTRY_UNKNOWN_TEXT );
			}

			// Uncompressed Size
			attributes.addAttribute( URI, ZIP_ENTRY_UNCOMP_SIZE_ATTR_NAME, 
						ZIP_ENTRY_UNCOMP_SIZE_ATTR_NAME, "CDATA", 
						"" + zipEntry.getSize() );

			// CRC32 Checksum
			attributes.addAttribute( URI, ZIP_ENTRY_CRC_ATTR_NAME, 
						ZIP_ENTRY_CRC_ATTR_NAME, "CDATA", 
						"" + zipEntry.getCrc() );

			// Comment
			if ( zipEntry.getComment() != null )
			{
				attributes.addAttribute( URI, ZIP_ENTRY_COMMENT_ATTR_NAME, 
							ZIP_ENTRY_COMMENT_ATTR_NAME, "CDATA", 
							"" + zipEntry.getComment() );
			}

			// Extra Data
			if ( zipEntry.getExtra() != null )
			{
				attributes.addAttribute( URI, ZIP_ENTRY_EXTRA_DATA_ATTR_NAME, 
							ZIP_ENTRY_EXTRA_DATA_ATTR_NAME, "CDATA", 
							"" + zipEntry.getExtra() );
			}

			// Modified Time (as long)
			attributes.addAttribute( URI, ZIP_ENTRY_MODIFIED_ATTR_NAME, 
						ZIP_ENTRY_MODIFIED_ATTR_NAME, "CDATA", 
						Long.toString( zipEntry.getTime() ) );

			// Modified Time (as date)
			attributes.addAttribute( URI, ZIP_ENTRY_DATE_ATTR_NAME, 
						ZIP_ENTRY_DATE_ATTR_NAME, "CDATA", 
						dateFormatter.format( new Date( zipEntry.getTime() ) ) );

			// Is directory indication
			if ( zipEntry.isDirectory() )
			{
				
				attributes.addAttribute( URI, ZIP_ENTRY_DIR_ATTR_NAME, 
							ZIP_ENTRY_DIR_ATTR_NAME, "CDATA", 
							"true" );
			}
					
			// Write the zipEntry node	
			super.contentHandler.startElement(URI, 
					ZIP_FILE_ZIPENTRY_NODE_NAME, 
					PREFIX + ':' + ZIP_FILE_ZIPENTRY_NODE_NAME, attributes );
			super.contentHandler.endElement(URI, 
					ZIP_FILE_ZIPENTRY_NODE_NAME, 
					PREFIX + ':' + ZIP_FILE_ZIPENTRY_NODE_NAME );
					
		}
		
    }

}
